/** Runs the given pre-parse hook, if defined, to modify the command line arguments. Returns the modified command line arguments. */
export default function runPreParseHook({ args, preParseHook, fail }: {
  args: string[],
  preParseHook?: (args: string[], fail: (message: string) => void) => (string[] | undefined),
  fail: (errorMessage: string) => never,
}): string[] {
  if (preParseHook === undefined) {
    return args;
  }

  try {
    const modifiedArgs = preParseHook(args, fail);
    return modifiedArgs !== undefined ? modifiedArgs : args;
  } catch (error) {
    return fail(`Could not run pre-parse hook, encountered error: ${String(error)}.`);
  }
}
