import IOption from './IOption';

export default function verifyAndStoreOptionValue({
  option,
  value,
  verify,
  message,
  fail,
  outputArgs,
  requiredOptions,
}: {
  option: string,
  value: string,
  verify: IOption['verify'],
  message: IOption['message'],
  fail: (errorMessage: string) => never,
  outputArgs: { [key: string]: string },
  requiredOptions: { [key: string]: boolean },
}) {
  //Verify value for correctness
  try {
    if (verify !== undefined && !verify(value)) {
      if (message !== undefined) {
        try {
          const customMessage = message(value);
          if (customMessage !== undefined && customMessage !== '') {
            return fail(customMessage);
          }
        } catch (error) {
          return fail(`Invalid value set for option "${option}": "${value}". Could not show custom error message: ${String(error)}`);
        }
      }
      return fail(`Invalid value set for option "${option}": "${value}".`);
    }
  } catch (error) {
    return fail(`Invalid value set for option "${option}", the validation of "${value}" failed: ${String(error)}`);
  }

  //Remember the value, and if it is a required argument, mark that we have encountered it
  outputArgs[option] = value;
  if (requiredOptions[option] === false) {
    requiredOptions[option] = true;
  }
}
